! LHAGLSTA common block
!implicit none
double precision xminnum, xmaxnum
double precision q2minnum, q2maxnum
double precision totnum
double precision xminnup, xmaxnup
double precision q2minnup, q2maxnup
double precision totnup
common /lhaglsta/ xminnum, xmaxnum, q2minnum, q2maxnum, totnum, xminnup, xmaxnup, q2minnup, q2maxnup, totnup
save /lhaglsta/
