! Shortcut include for all LHAPDF steering common blocks
!include 'parmsetup.inc'
include 'commonlhacontrol.inc'
include 'commonlhaglsta.inc'
include 'commonlhapdfc.inc'
include 'commonlhapdf.inc'
include 'commonlhasets.inc'
